import math
from itertools import groupby
from math import pi, cos

EARTH = 6378.137  # Radius of the earth in kilometer
METER = (1 / ((2 * pi / 360) * EARTH)) / 1000  # 1 meter in degree

# find real world coordinates to lidar
#
# p - area using real world coordinate
# l - area using lidar coordinate(pointcloud)
# max_limit - maximum area limit of a lidar scann in real world coordinates
def position_relative_to_pointcloud(l1, l2, p1, p2, center_l, center_p, max_lim):
    return [
        ((center_p[0] - p1[0]) * ((l1[0] - l2[0]) / (max_lim[1][0] - max_lim[3][0])) + center_l[0],
            (center_p[1] - p1[1]) * ((l1[1] - l2[1]) / (max_lim[1][1] - max_lim[3][1])) + center_l[1]),
        ((center_p[0] - p2[0]) * ((l1[0] - l2[0]) / (max_lim[1][0] - max_lim[3][0])) + center_l[0],
            (center_p[1] - p2[1]) * ((l1[1] - l2[1]) / (max_lim[1][1] - max_lim[3][1])) + center_l[1])
    ]
    #return [(((min[0] - max[0]) * ((start[0] - limit[1][0]) / (limit[1][0] - limit[3][0]))) + center[0],
    #        (((min[1] - max[1]) * ((start[1] - limit[1][0]) / (limit[1][1] - limit[3][1]))) + center[1])),
    #        ((((min[0] - max[0]) * ((end[0] - limit[1][0]) / (limit[1][0] - limit[3][0]))) + center[0],
    #        (((min[1] - max[1]) * ((end[1] - limit[1][0]) / (limit[1][1] - limit[3][1])))) + center[1]))]

print(position_relative_to_pointcloud((-5,5), (-2,2), (1.2,1.6), (1.8,1.4), (-3.5,3.5), (1.5,1.5), [(0,0),(1,2),(0,0),(2,1)]))

# check if lidar points is within range of the area selected
def inArea(position, areaRange):
    x, y, _ = position # position to be checked
    #print((areaRange[0][0])," < ",x," < ",(areaRange[1][0])," and ",(areaRange[0][1])," > ",(y)," > ",(areaRange[1][1])," ",((areaRange[0][0]) < x < (areaRange[1][0])) and ((areaRange[0][1]) > (y) > (areaRange[1][1])))
    if ((areaRange[0][0]) < x < (areaRange[1][0])) and ((areaRange[0][1]) > y > (areaRange[1][1])):
        return True
    else:
        return False

# find distance between two points
def distance(point1, point2):
    y1, x1 = point1
    y2, x2 = point2
    return math.sqrt(abs(y2 - y1)**2 + abs(x2 - x1)**2)

# find the closest point in json list
def closest_points(point, list, coords_taken):
    closest_point_found = None
    closest_dist = float('inf')
    for current_point in list:
        dist = distance(point, current_point['properties']['sub_div_center'])
        if dist < closest_dist and current_point not in coords_taken:
            closest_dist = dist
            closest_point_found = current_point
    return closest_point_found

# Calculation of height in an area from las files
def find_height(points):
    print(points, " 5 ")
    height_differences = [] # final container for height data

    # sort the points
    sorted_coords = sorted(points, key=lambda coord: (coord[0], coord[1]))

    # group the sorted points that has the same xy- coordinates together
    groupCoords = [list(group) for key, group in groupby(sorted_coords, key=lambda coord: (coord[0], coord[1]))]

    # loop through the groups to find the difference in height
    for group in groupCoords:
        if len(group) < 2 or group[0] == group[1]:
            continue # jump over iteration if there is only one coordinate or lidar registered the same point twice

        # find max and min height
        min_height = min(coords[2] for coords in group)
        max_height = max(coords[2] for coords in group)

        # difference between them
        difference = max_height - min_height
        height_differences.append(difference)

    # list of thickness in an area
    return height_differences

def calculate_corners(lat, lng, area_offset):
    """Calculate corners of polygon based on a center coordinate

    Arguments:
    lat -- center latitude
    lng -- center longitude
    """
    # From https://stackoverflow.com/questions/7477003/calculating-new-longitude-latitude-from-old-n-meters
    # Formulas:
    offset_lng, offset_lat = area_offset
    lat_pos = lat + (offset_lat * METER)
    lng_pos = lng + (offset_lng * METER) / cos(lat * (pi / 180))

    lat_neg = lat - (offset_lat * METER)
    lng_neg = lng - (offset_lng * METER) / cos(lat * (pi / 180))

    return [
        (lat_neg, lng_pos), # top left
        (lat_pos, lng_pos), # top right
        (lat_pos, lng_neg), # bottom right
        (lat_neg, lng_neg)  # bottom left
    ]

# separate the zones into smaller area, into a grid
def define_gridareas(lat, lng, area_offset, grid_size):
    # container for an area turned into a grid of areas
    grided_area = []

    # find the main area's corner positions
    main_area = calculate_corners(lat, lng, area_offset)

    offset_lng, offset_lat = area_offset

    # find the main area's range size
    area_size = (main_area[2][0] - main_area[0][0], main_area[0][1] - main_area[2][1])
    # find each subareas vector to it's center
    dist_to_subcenter = (area_size[0]/(grid_size*2), area_size[1]/(grid_size*2))

    # find subareas size relative to main area
    subarea_offset_lng = offset_lng/grid_size
    subarea_offset_lat = offset_lat/grid_size

    group_id = 0
    i = 0
    # find the center coordinates of each area in grid to find the corner areas
    for y in (range(grid_size)):
        relative_size_lat = y / grid_size
        for x in (range(grid_size)):
            relative_size_lng = x / grid_size
            lat_pos = main_area[3][0] + relative_size_lat * area_size[0] + dist_to_subcenter[0]
            lng_pos = main_area[3][1] + relative_size_lng * area_size[1] + dist_to_subcenter[1]
            # id of each subarea
            #sub_id = x % grid_size + grid_size * y

            if (x+(y*grid_size)) % ((grid_size*2) + 1) == grid_size * 2:
                group_id += 2

            if (x+(y*grid_size)) % 2 == 0 and (x+(y*grid_size)) != 0:
                i += 1

            # use the center of sub areas to find the corner of each subarea
            subarea_offset = (subarea_offset_lng, subarea_offset_lat)
            corners = calculate_corners(lat_pos, lng_pos, subarea_offset)
            grided_area.append((group_id + i%2, corners))

    return grided_area
#[1,2,2,3,4,5,6,3,4,6,8,9,5,3,5.7,8,5,3]
#print(define_gridareas(60,10,1500,4))
#print(define_gridareas(3435693.5,6299200.0, 400000,4))