import numpy as np
import laspy
import json
import math
import utm # src: https://github.com/Turbo87/utm
from server.data_processing.area_processing import calculate_corners, define_gridareas, inArea, find_height, closest_points, position_relative_to_pointcloud

import matplotlib.pyplot as plt

# hard coded files for test datas
lazData_path = ["server/example_lidar_data/ot_N_000005_1.laz", "server/example_lidar_data/ot_N_000033_1.laz"]

# Info about data
def about_laz_file():
    with laspy.open(lazData_path[0]) as fh:
        # Print metadata properties
        print("File Version:", fh.header.version)
        print("Point Count:", fh.header.point_count)
        print("Scale Factors:", fh.header.scale)
        print("Offset:", fh.header.offset)

        las = fh.read()
        print(las)
        print('Points from data:', len(las.points))
        ground_pts = las.classification == 2
        bins, counts = np.unique(las.return_number[ground_pts], return_counts=True)
        print('Ground Point Return Number distribution:')
        for r, c in zip(bins, counts):
            print('    {}:{}'.format(r, c))

    return [las.header.version, las.header.point_count, las.header.scale, las.header.offset]

# find the height of an area based on the coordinates of it's center
# and it's affiliations (subId and groupId) (soon to be implemented
def calculate_area_data(center, body_of_water):
    # container for all the heights in area
    area_heights = []

    # zone coords taken
    taken_coords = []

    # read json data with path to data for specific water body
    file_name = "server/map_handler/lake_relations/" + body_of_water + "_div.json"
    with open(file_name) as data:
        map_data = json.load(data)

    # grid cell offset
    #cell_x, cell_y = map_data['tile_width'], map_data['tile_height']
    cell_x, cell_y = (2000, 1000)

    # convert the offset to meter
    cell_x = 111.320 * cell_x
    cell_y = 111.320 * math.cos(60) * cell_y

    # set the limit of the area compared to local coordinates
    area_limit = calculate_corners(center[0], center[1], (cell_x, cell_y))
    #area_limit = calculate_corners(sub_center[0], sub_center[1], (cell_x/4, cell_y/4))

    # grid data
    map_data = map_data['features']
    map_zones = [area_limit[1], area_limit[3]]
    map_data = list(filter(lambda point_position: inArea((point_position['properties']['sub_div_center'][0], point_position['properties']['sub_div_center'][1], 0.0), map_zones), map_data))

    # Refactor lidar data to a readable format
    iceOver = laspy.read(lazData_path[0])
    iceUnder = laspy.read(lazData_path[1])

    # add two files together temporary test data(soon to be removed)
    ice_points = list(zip(iceOver.X,iceOver.Y,iceOver.Z)) + list(zip(iceUnder.X,iceUnder.Y,iceUnder.Z))
    ice_points = list(filter(lambda point_position: inArea((point_position[0],point_position[1],0.0),[(-3300000,5000000), (-3200000,4500000)]),ice_points))

    # only for visualizating the testing data
    #print("max",max(ice_points))
    #print("min",min(ice_points))

    #x = [points[0] for points in ice_points]
    #y = [points[1] for points in ice_points]

    # Plot the points
    #plt.plot(x, y, 'bo')  # 'bo' specifies blue color and circle markers
    #plt.xlabel('X axis')
    #plt.ylabel('Y axis')
    #plt.title('Plotting Points')
    #plt.grid(True)  # Add grid
    #plt.show()

    max_point = max(ice_points)
    min_point = min(ice_points)

    # define all the sub-areas within the area, local coordinates
    grid_area_heights = define_gridareas(center[0], center[1], (cell_x, cell_y),4)
    # find the heights of each sub-area => area-heights
    if len(map_data) > 0:
        for sub_area in grid_area_heights:
            start = (sub_area[1][1])
            end = (sub_area[1][3])

            #test data
            # zone coordinates sett to be relative to the lidar data's point cloud
            areazone = position_relative_to_pointcloud(min_point, max_point, start, end, center, area_limit)

            # calculate map zones height
            ys, xs = start
            ye, xe = end
            sub_center = ((xs + xe)/2, (ys + ye)/2)
            # check if area is part of water body
            part_of_subarea_of_waterbody = list(filter(lambda pos: inArea((pos['properties']['sub_div_center'][0], pos['properties']['sub_div_center'][1], 0.0), [start,end]), map_data))
            if(len(part_of_subarea_of_waterbody) > 0):
                current_map_zone = closest_points(sub_center, part_of_subarea_of_waterbody, taken_coords)
                sub_center = current_map_zone['properties']['sub_div_center']
                taken_coords.append(sub_center)
                print("item added", sub_center, " len ", len(taken_coords))
            else:
                print("nothing here")
                continue

            current_zone_id = current_map_zone['properties']['sub_div_id']

            # filter data within sub-area zones
            print(areazone[0][0]," < ",ice_points[0][0]," < ",areazone[1][0])
            print(areazone[0][1]," > ",ice_points[0][1]," > ",areazone[1][1])
            points_in_area = list(filter(lambda point_position: inArea(point_position, areazone), ice_points))

            if(len(points_in_area) > 0):
                heights = find_height(points_in_area)

                if current_map_zone is not None:
                    area_heights.append((current_zone_id, sub_area[0], sub_center, heights))
            else:
                print("no points in area")
        return area_heights
    else:
        return [] # return [0] if no data collected from lidar


print(calculate_area_data((61, 11), 'mj\u00f8sa'))


