import json
import os
from datetime import datetime
from server.data_processing.process_lidar_data import calculate_area_data, about_laz_file

# input_new_Lidar_data send new data gathered from the lidar and send it to the database (from the drone, most likely)
def input_new_Lidar_data(self, cursor, sensorId, bodyOfWater):
    try:
        print("name=",bodyOfWater)
        # hard coded coordinates
        latitude = 60.816848
        longitude = 10.723823

        total_measurement_average = 0  # the total average of a measurement

        # data about the file read from
        about_laz = about_laz_file()
        scale_factor = max(about_laz[2])

        # create a new measurement with the time the data is sent, sensor type, where
        # and an estimate of average thickness of ice on water body
        cursor.execute('''
            INSERT INTO Measurement(  SensorID, TimeMeasured, WaterBodyName, 
                                        WholeAverageThickness, CenterLat, CenterLon) VALUES 
                (?,?,?,?,?,?);
        ''', (sensorId, datetime.utcnow().replace(microsecond=0), bodyOfWater, 0, latitude, longitude))

        # auto generate new measurement id
        measurement_id = cursor.lastrowid

        # calculate the area of to be calculated based on the coordinates given to the calculation model
        areas_data = calculate_area_data((latitude, longitude), bodyOfWater)
        print(bodyOfWater)
        print(len(areas_data))
        lidar_json_data = []

        if(areas_data):
            # store lidar data in jason formate
            # calculate data for each zone within the area
            for area in areas_data:
                # lng and lat relative to map
                map_lat, map_lng = area[2]

                if(len(area[3]) != 0):
                    average = sum(area[3])/len(area[3])
                    minimum_thickness = min(area[3])
                else:
                    average = 0
                    minimum_thickness = 0

                total_measurement_average += average

                # input the data into the database
                cursor.execute('''
                INSERT INTO SubDivision(MeasurementID, SubDivisionID, GroupID, MinimumThickness, AverageThickness, CenterLatitude, CenterLongitude, Accuracy) VALUES
                    (?,?,?,?,?,?,?,?);
                ''', (measurement_id, area[0], area[1], float(minimum_thickness), float(average), float(map_lat), float(map_lng), scale_factor))

                # set up json formate
                lidar_read = {
                    'MeasurementId': measurement_id,
                    'SubId': area[0],
                    'GroupId': area[1],
                    'SubCenter': (map_lat, map_lng),
                    'Heights': area[3]
                }

                lidar_json_data.append(lidar_read)

            total_measurement_average = total_measurement_average / len(areas_data)

            # input the newly generated measurement_id and whole average thickness
            cursor.execute('''
                UPDATE Measurement
                SET measurementID = ?, WholeAverageThickness = ?
                WHERE MeasurementID IS NULL AND WholeAverageThickness = 0;
            ''', (int(measurement_id), total_measurement_average), )
        else:
            print('No data found, line 79')

        # send the changes to the database
        cursor.connection.commit()

        # Send response
        self.send_response(200)
        self.send_header('Content-type', "application/json")
        self.end_headers()

        file_path = "./lake_relation/newest_lidar_data.json"
        content = None

        if len(lidar_json_data) > 0:
            if os.path.exists(file_path):
                os.remove(file_path)

            # convert list of lidar data to json
            content = json.dumps(lidar_json_data)
            with open(file_path, "w") as file:
                file.write(content)
        else:
            print('No data found, line 101')
            content = json.dumps([])

        # Write content data to response object
        self.wfile.write(content.encode('utf-8'))

    # error handling
    except Exception as e:
        print("An error occurred", e)
        # rollback in case of error
        cursor.connection.rollback()